@@ -14,7 +14,6 @@ use ide::{
     SourceChange, TextEdit,
 };
 use ide_db::SymbolKind;
-use itertools::Itertools;
 use lsp_server::ErrorCode;
 use lsp_types::{
     CallHierarchyIncomingCall, CallHierarchyIncomingCallsParams, CallHierarchyItem,
@@ -109,7 +108,7 @@ pub(crate) fn handle_syntax_tree(
     let _p = profile::span("handle_syntax_tree");
     let id = from_proto::file_id(&snap, &params.text_document.uri)?;
     let line_index = snap.file_line_index(id)?;
-    let text_range = params.range.map(|r| from_proto::text_range(&line_index, r));
+    let text_range = params.range.and_then(|r| from_proto::text_range(&line_index, r).ok());
     let res = snap.analysis.syntax_tree(id, text_range)?;
     Ok(res)
 }
@@ -150,7 +149,7 @@ pub(crate) fn handle_expand_macro(
     let _p = profile::span("handle_expand_macro");
     let file_id = from_proto::file_id(&snap, &params.text_document.uri)?;
     let line_index = snap.file_line_index(file_id)?;
-    let offset = from_proto::offset(&line_index, params.position);
+    let offset = from_proto::offset(&line_index, params.position)?;
 
     let res = snap.analysis.expand_macro(FilePosition { file_id, offset })?;
     Ok(res.map(|it| lsp_ext::ExpandedMacro { name: it.name, expansion: it.expansion }))
@@ -167,7 +166,7 @@ pub(crate) fn handle_selection_range(
         .positions
         .into_iter()
         .map(|position| {
-            let offset = from_proto::offset(&line_index, position);
+            let offset = from_proto::offset(&line_index, position)?;
             let mut ranges = Vec::new();
             {
                 let mut range = TextRange::new(offset, offset);
@@ -206,19 +205,20 @@ pub(crate) fn handle_matching_brace(
     let _p = profile::span("handle_matching_brace");
     let file_id = from_proto::file_id(&snap, &params.text_document.uri)?;
     let line_index = snap.file_line_index(file_id)?;
-    let res = params
+    params
         .positions
         .into_iter()
         .map(|position| {
             let offset = from_proto::offset(&line_index, position);
-            let offset = match snap.analysis.matching_brace(FilePosition { file_id, offset }) {
-                Ok(Some(matching_brace_offset)) => matching_brace_offset,
-                Err(_) | Ok(None) => offset,
-            };
-            to_proto::position(&line_index, offset)
+            offset.map(|offset| {
+                let offset = match snap.analysis.matching_brace(FilePosition { file_id, offset }) {
+                    Ok(Some(matching_brace_offset)) => matching_brace_offset,
+                    Err(_) | Ok(None) => offset,
+                };
+                to_proto::position(&line_index, offset)
+            })
         })
-        .collect();
-    Ok(res)
+        .collect()
 }
 
 pub(crate) fn handle_join_lines(
@@ -233,7 +233,7 @@ pub(crate) fn handle_join_lines(
 
     let mut res = TextEdit::default();
     for range in params.ranges {
-        let range = from_proto::text_range(&line_index, range);
+        let range = from_proto::text_range(&line_index, range)?;
         let edit = snap.analysis.join_lines(&config, FileRange { file_id, range })?;
         match res.union(edit) {
             Ok(()) => (),
@@ -275,9 +275,7 @@ pub(crate) fn handle_on_type_formatting(
     let char_typed = params.ch.chars().next().unwrap_or('\0');
 
     let text = snap.analysis.file_text(position.file_id)?;
-    if !text[usize::from(position.offset)..].starts_with(char_typed) {
-        // Add `always!` here once VS Code bug is fixed:
-        //   https://github.com/rust-analyzer/rust-analyzer/issues/10002
+    if stdx::never!(!text[usize::from(position.offset)..].starts_with(char_typed)) {
         return Ok(None);
     }
 
@@ -534,7 +532,7 @@ pub(crate) fn handle_will_rename_files(
     let mut source_change = source_changes.next().unwrap_or_default();
     source_change.file_system_edits.clear();
     // no collect here because we want to merge text edits on same file ids
-    source_change.extend(source_changes.map(|it| it.source_file_edits).flatten());
+    source_change.extend(source_changes.flat_map(|it| it.source_file_edits));
     if source_change.source_file_edits.is_empty() {
         Ok(None)
     } else {
@@ -676,7 +674,7 @@ pub(crate) fn handle_runnables(
     let _p = profile::span("handle_runnables");
     let file_id = from_proto::file_id(&snap, &params.text_document.uri)?;
     let line_index = snap.file_line_index(file_id)?;
-    let offset = params.position.map(|it| from_proto::offset(&line_index, it));
+    let offset = params.position.and_then(|it| from_proto::offset(&line_index, it).ok());
     let cargo_spec = CargoTargetSpec::for_file(&snap, file_id)?;
 
     let expect_test = match offset {
@@ -840,7 +838,7 @@ pub(crate) fn handle_completion_resolve(
 
     let file_id = from_proto::file_id(&snap, &resolve_data.position.text_document.uri)?;
     let line_index = snap.file_line_index(file_id)?;
-    let offset = from_proto::offset(&line_index, resolve_data.position.position);
+    let offset = from_proto::offset(&line_index, resolve_data.position.position)?;
 
     let additional_edits = snap
         .analysis
@@ -854,7 +852,7 @@ pub(crate) fn handle_completion_resolve(
         )?
         .into_iter()
         .flat_map(|edit| edit.into_iter().map(|indel| to_proto::text_edit(&line_index, indel)))
-        .collect_vec();
+        .collect::<Vec<_>>();
 
     if !all_edits_are_disjoint(&original_completion, &additional_edits) {
         return Err(LspError::new(
@@ -897,13 +895,12 @@ pub(crate) fn handle_signature_help(
 ) -> Result<Option<lsp_types::SignatureHelp>> {
     let _p = profile::span("handle_signature_help");
     let position = from_proto::file_position(&snap, params.text_document_position_params)?;
-    let call_info = match snap.analysis.call_info(position)? {
+    let help = match snap.analysis.signature_help(position)? {
         Some(it) => it,
         None => return Ok(None),
     };
     let concise = !snap.config.call_info_full();
-    let res =
-        to_proto::signature_help(call_info, concise, snap.config.signature_help_label_offsets());
+    let res = to_proto::signature_help(help, concise, snap.config.signature_help_label_offsets());
     Ok(Some(res))
 }
 
@@ -1090,7 +1087,7 @@ pub(crate) fn handle_code_action(
             .ranges
             .iter()
             .copied()
-            .map(|range| from_proto::text_range(&line_index, range))
+            .filter_map(|range| from_proto::text_range(&line_index, range).ok())
             .any(|fix_range| fix_range.intersect(frange.range).is_some());
         if intersect_fix_range {
             res.push(fix.action.clone());
@@ -1112,7 +1109,7 @@ pub(crate) fn handle_code_action_resolve(
 
     let file_id = from_proto::file_id(&snap, &params.code_action_params.text_document.uri)?;
     let line_index = snap.file_line_index(file_id)?;
-    let range = from_proto::text_range(&line_index, params.code_action_params.range);
+    let range = from_proto::text_range(&line_index, params.code_action_params.range)?;
     let frange = FileRange { file_id, range };
 
     let mut assists_config = snap.config.assist();
@@ -1164,7 +1161,7 @@ pub(crate) fn handle_code_action_resolve(
 }
 
 fn parse_action_id(action_id: &str) -> Result<(usize, SingleResolve), String> {
-    let id_parts = action_id.split(':').collect_vec();
+    let id_parts = action_id.split(':').collect::<Vec<_>>();
     match id_parts.as_slice() {
         [assist_id_string, assist_kind_string, index_string] => {
             let assist_kind: AssistKind = assist_kind_string.parse()?;
@@ -1320,13 +1317,25 @@ pub(crate) fn handle_inlay_hints(
     params: InlayHintsParams,
 ) -> Result<Vec<InlayHint>> {
     let _p = profile::span("handle_inlay_hints");
-    let file_id = from_proto::file_id(&snap, &params.text_document.uri)?;
+    let document_uri = &params.text_document.uri;
+    let file_id = from_proto::file_id(&snap, document_uri)?;
     let line_index = snap.file_line_index(file_id)?;
+    let range = params
+        .range
+        .map(|range| {
+            from_proto::file_range(
+                &snap,
+                TextDocumentIdentifier::new(document_uri.to_owned()),
+                range,
+            )
+        })
+        .transpose()?;
+    let inlay_hints_config = snap.config.inlay_hints();
     Ok(snap
         .analysis
-        .inlay_hints(&snap.config.inlay_hints(), file_id)?
+        .inlay_hints(&inlay_hints_config, file_id, range)?
         .into_iter()
-        .map(|it| to_proto::inlay_hint(&line_index, it))
+        .map(|it| to_proto::inlay_hint(inlay_hints_config.render_colons, &line_index, it))
         .collect())
 }
 
